<?php
/**
 * Plugin Name:         RS Post Archive
 * Description:         Archive posts without removing or un-publishing them.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-post-archive/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-post-archive
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             0.5.1
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


add_action( 'plugins_loaded', function() {
    class PostArchive extends PluginBase {
        protected $plugin_file = __FILE__;

        use PluginBaseSettings;

        /*************************************************************
         *
         * Post type for archived posts
         *
         *************************************************************/

        function do_init_action_archive_post_type() {
            register_post_type(
                'post-archive',
                [
                    'labels'                => [
                        'name'                  => __( 'Archived posts', 'rs-post-archive' ),
                        'singular_name'         => __( 'Archived post', 'rs-post-archive' ),
                        'all_items'             => __( 'All Archived posts', 'rs-post-archive' ),
                        'archives'              => __( 'Archived post Archives', 'rs-post-archive' ),
                        'attributes'            => __( 'Archived post Attributes', 'rs-post-archive' ),
                        'insert_into_item'      => __( 'Insert into Archived post', 'rs-post-archive' ),
                        'uploaded_to_this_item' => __( 'Uploaded to this Archived post', 'rs-post-archive' ),
                        'featured_image'        => _x( 'Featured Image', 'rs-post-archive', 'rs-post-archive' ),
                        'set_featured_image'    => _x( 'Set featured image', 'rs-post-archive', 'rs-post-archive' ),
                        'remove_featured_image' => _x( 'Remove featured image', 'rs-post-archive', 'rs-post-archive' ),
                        'use_featured_image'    => _x( 'Use as featured image', 'rs-post-archive', 'rs-post-archive' ),
                        'filter_items_list'     => __( 'Filter Archived posts list', 'rs-post-archive' ),
                        'items_list_navigation' => __( 'Archived posts list navigation', 'rs-post-archive' ),
                        'items_list'            => __( 'Archived posts list', 'rs-post-archive' ),
                        'new_item'              => __( 'New Archived post', 'rs-post-archive' ),
                        'add_new'               => __( 'Add New', 'rs-post-archive' ),
                        'add_new_item'          => __( 'Add New Archived post', 'rs-post-archive' ),
                        'edit_item'             => __( 'Edit Archived post', 'rs-post-archive' ),
                        'view_item'             => __( 'View Archived post', 'rs-post-archive' ),
                        'view_items'            => __( 'View Archived posts', 'rs-post-archive' ),
                        'search_items'          => __( 'Search Archived posts', 'rs-post-archive' ),
                        'not_found'             => __( 'No Archived posts found', 'rs-post-archive' ),
                        'not_found_in_trash'    => __( 'No Archived posts found in trash', 'rs-post-archive' ),
                        'parent_item_colon'     => __( 'Parent Archived post:', 'rs-post-archive' ),
                        'menu_name'             => __( 'Archived posts', 'rs-post-archive' ),
                    ],
                    'public'                => true,
                    'hierarchical'          => false,
                    'show_ui'               => true,
                    'show_in_nav_menus'     => true,
                    'supports'              => [ "title", "editor", "author", "thumbnail", "page-attributes", "custom-fields", "revisions", "excerpt" ],
                    'has_archive'           => true,
                    'rewrite'               => true,
                    'query_var'             => true,
                    'menu_position'         => null,
                    'menu_icon'             => 'dashicons-archive',
                    'show_in_rest'          => true,
                    'rest_base'             => 'post-archive',
                    'rest_controller_class' => 'WP_REST_Posts_Controller',
                ]
            );

        }

        /**
         * Sets the post updated messages for the 'post-archive' post type.
         *
         * @param  array $messages Post updated messages.
         * @return array Messages for the 'post-archive' post type.
         */

        function do_post_updated_messages_filter_archive_post_type( $messages ) {
            global $post;

            $permalink = get_permalink( $post );

            $messages['post-archive'] = [
                0  => '', // Unused. Messages start at index 1.
                /* translators: %s: post permalink */
                1  => sprintf( __( 'Archived post updated. <a target="_blank" href="%s">View Archived post</a>', 'rs-post-archive' ), esc_url( $permalink ) ),
                2  => __( 'Custom field updated.', 'rs-post-archive' ),
                3  => __( 'Custom field deleted.', 'rs-post-archive' ),
                4  => __( 'Archived post updated.', 'rs-post-archive' ),
                /* translators: %s: date and time of the revision */
                5  => isset( $_GET['revision'] ) ? sprintf( __( 'Archived post restored to revision from %s', 'rs-post-archive' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false, // phpcs:ignore WordPress.Security.NonceVerification.Recommended
                /* translators: %s: post permalink */
                6  => sprintf( __( 'Archived post published. <a href="%s">View Archived post</a>', 'rs-post-archive' ), esc_url( $permalink ) ),
                7  => __( 'Archived post saved.', 'rs-post-archive' ),
                /* translators: %s: post permalink */
                8  => sprintf( __( 'Archived post submitted. <a target="_blank" href="%s">Preview Archived post</a>', 'rs-post-archive' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
                /* translators: 1: Publish box date format, see https://secure.php.net/date 2: Post permalink */
                9  => sprintf( __( 'Archived post scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview Archived post</a>', 'rs-post-archive' ), date_i18n( __( 'M j, Y @ G:i', 'rs-post-archive' ), strtotime( $post->post_date ) ), esc_url( $permalink ) ),
                /* translators: %s: post permalink */
                10 => sprintf( __( 'Archived post draft updated. <a target="_blank" href="%s">Preview Archived post</a>', 'rs-post-archive' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
            ];

            return $messages;
        }


        /**
         * Sets the bulk post updated messages for the 'post-archive' post type.
         *
         * @param  array $bulk_messages Arrays of messages, each keyed by the corresponding post type. Messages are
         *                              keyed with 'updated', 'locked', 'deleted', 'trashed', and 'untrashed'.
         * @param  int[] $bulk_counts   Array of item counts for each message, used to build internationalized strings.
         * @return array Bulk messages for the 'post-archive' post type.
         */

        protected $bulk_post_updated_messages_callback_arguments = 2;

        function do_bulk_post_updated_messages_filter_archived_post_type( $bulk_messages, $bulk_counts ) {
            global $post;

            $bulk_messages['post-archive'] = [
                /* translators: %s: Number of Archived posts. */
                'updated'   => _n( '%s Archived post updated.', '%s Archived posts updated.', $bulk_counts['updated'], 'rs-post-archive' ),
                'locked'    => ( 1 === $bulk_counts['locked'] ) ? __( '1 Archived post not updated, somebody is editing it.', 'rs-post-archive' ) :
                /* translators: %s: Number of Archived posts. */
                _n( '%s Archived post not updated, somebody is editing it.', '%s Archived posts not updated, somebody is editing them.', $bulk_counts['locked'], 'rs-post-archive' ),
                /* translators: %s: Number of Archived posts. */
                'deleted'   => _n( '%s Archived post permanently deleted.', '%s Archived posts permanently deleted.', $bulk_counts['deleted'], 'rs-post-archive' ),
                /* translators: %s: Number of Archived posts. */
                'trashed'   => _n( '%s Archived post moved to the Trash.', '%s Archived posts moved to the Trash.', $bulk_counts['trashed'], 'rs-post-archive' ),
                /* translators: %s: Number of Archived posts. */
                'untrashed' => _n( '%s Archived post restored from the Trash.', '%s Archived posts restored from the Trash.', $bulk_counts['untrashed'], 'rs-post-archive' ),
            ];

            return $bulk_messages;
        }


        /************************************************************************
         *
         *	Connect post taxonomies to archived posts
         *
         ************************************************************************/

        function do_wp_loaded_action() {
            $taxonomies = get_object_taxonomies( 'post' );

            foreach ( $taxonomies as $taxonomy )
                register_taxonomy_for_object_type( $taxonomy, 'post-archive' );
        }

        /************************************************************
         *
         * Fix searches on taxonomy archives to include archived posts
         * Fix author archives to include archive posts
         * Fix global feed to include archived posts
         *
         ************************************************************/

        function do_pre_get_posts_action_archives( $wp_query ) {
            if ( $wp_query->is_main_query() && ! is_admin() ) {
                $this->integrate_post_type_in_archives( $wp_query, 'post-archive', $this->get_option( 'taxonomy_searches' ) );
            }
        }


        /************************************************************************
         *
         *	Archive post row/bulk actions
         *
         ************************************************************************/

        // Row actions shared post / post-archive

        protected $post_row_actions_callback_arguments = 2;

        function do_post_row_actions_filter_archive( $actions, $post ) {
            if ( in_array( $post->post_type, [ 'post', 'post-archive' ] ) ) {
                $nonce = wp_create_nonce( 'post-archive-action-nonce' );
                $return = urlencode( site_url( $_SERVER['REQUEST_URI'] ) );

                if ( 'post' === $post->post_type ) {
                    $action = 'archive_post';
                    $label = __( 'Archive', 'rs-post-archive' );
                }
                else if ( 'post-archive' == $post->post_type ) {
                    $action = 'unarchive_post';
                    $label = __( 'Un-archive', 'rs-post-archive' );
                }

                $url = $this->build_admin_url( 'admin-post.php', [
                    'action' => $action,
                    'post' => $post->ID,
                    '_wpnonce' => $nonce,
                    'return' => $return,
                ] );

                $actions['archive-post'] = sprintf( '<a href="%1$s">%2$s</a>',
                                                    esc_url( $url ),
                                                    esc_html( $label )
                );
            }

            return $actions;
        }

        // Handler for archive actions on posts

        function do_admin_post_archive_post_action() {
            if ( !( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'post-archive-action-nonce' ) ) )
                die( __( 'Security check', 'rs-post-archive' ) );

            if ( isset( $_GET['post'] ) ) {
                $this->archive_post( intval( $_GET['post'] ) );
            }

            if ( ! isset( $_GET['return'] ) )
                die( __( 'No return parameter specified', 'rs-post-archive' ) );
            $return = urldecode( $_GET['return'] );
            wp_redirect( $return );
            exit;
        }


        // Handler for un-archive actions on archived posts

        function do_admin_post_unarchive_post_action() {
            if ( !( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'post-archive-action-nonce' ) ) )
                die( __( 'Security check', 'rs-post-archive' ) );

            if ( isset( $_GET['post'] ) ) {
                $this->unarchive_post( intval( $_GET['post'] ) );
            }

            if ( ! isset( $_GET['return'] ) )
                die( __( 'No return parameter specified', 'rs-post-archive' ) );
            $return = urldecode( $_GET['return'] );
            wp_redirect( $return );
            exit;
        }


        // Archive bulk action on posts

        protected $bulk_actions_edit_post_callback_hook = "bulk_actions-edit-post";

        function do_bulk_actions_edit_post_filter( $actions ) {
            $actions['post-archive'] = __('Archive', 'rs-post-archive');
            return $actions;
        }


        protected $handle_bulk_actions_edit_post_callback_hook = 'handle_bulk_actions-edit-post';
        protected $handle_bulk_actions_edit_post_callback_arguments = 3;

        function do_handle_bulk_actions_edit_post_filter( $redirect_url, $action, $post_ids ) {
            if ( $action == 'post-archive' ) {
                foreach ( $post_ids as $post_id ) {
                    $this->archive_post( $post_id );
                }
            }
            return $redirect_url;
        }



        // Un-archive bulk action on archived posts

        protected $bulk_actions_edit_post_archive_callback_hook = "bulk_actions-edit-post-archive";

        function do_bulk_actions_edit_post_archive_filter( $actions ) {
            $actions['post-unarchive'] = __('Un-archive', 'rs-post-archive');
            return $actions;
        }


        protected $handle_bulk_actions_edit_post_archive_callback_hook = 'handle_bulk_actions-edit-post-archive';
        protected $handle_bulk_actions_edit_post_archive_callback_arguments = 3;

        function do_handle_bulk_actions_edit_post_archive_filter( $redirect_url, $action, $post_ids ) {
            if ( $action == 'post-unarchive' ) {
                foreach ( $post_ids as $post_id ) {
                    $this->unarchive_post( $post_id );
                }
            }
            return $redirect_url;
        }


        // Actually archive or un-archive a post

        function archive_post( $post ) {
            $post = get_post( $post );

            error_log( sprintf( "Archiving post %d - %s", $post->ID, $post->post_title ) );

            if ( $post && 'post' == $post->post_type )
                wp_update_post( [
                    'ID' => $post->ID,
                    'post_type' => 'post-archive',
                ] );
        }


        function unarchive_post( $post ) {
            $post = get_post( $post );

            error_log( sprintf( "Un-archiving post %d - %s", $post->ID, $post->post_title ) );

            if ( $post && 'post-archive' == $post->post_type )
                wp_update_post( [
                    'ID' => $post->ID,
                    'post_type' => 'post',
                ] );
        }


        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'Post Archive', 'rs-post-archive'),
                __( 'RS Post Archive', 'rs-post-archive' )
            );
        }

        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_taxonomies',
                __( 'Taxonomies', 'rs-post-archive' ),
                __( 'Select where you want archived posts to appear.', 'rs-post-archive' ),
            );

            $this->settings_add_field(
                'taxonomy_searches', $section,
                __( 'Include in taxonomy searches', 'rs-post-archive' ),
                'settings_field_checkbox_html',
                [
                    'type' => 'checkbox',
                    'default' => false,
                ]
            );

            $this->settings_add_field(
                'author_searches', $section,
                __( 'Include in author pages', 'rs-post-archive' ),
                'settings_field_checkbox_html',
                [
                    'type' => 'checkbox',
                    'default' => false,
                ]
           );

        }
    }

    new PostArchive();
} );
